//
//  Gnuplot - a C++ class interface with Gnuplot
//  V2.0
//
//  Created by Richel Bilderbeek on Fri Jun 10 2005.
//  Open source.
//
//  Richel Bilderbeek
//  Vinkenstraat 54b
//  9713 TJ Groningen
//  The Netherlands
//  (0031)-(0)50-3115529
//  E-mail: richelbilderbeek@fastmail.fm
//
//  V2.0: 12th September 2005: Got program to compile under Windows environment.
//                             Program is found NOT to run under Windows.
//                             Somebody has an idea how to open Gnuplot.exe ???
//  V1.0: 10th June 2005: first version developed on Macintosh
//
//
//  This class functions well at a Macintosh. Under Windows, the pipe to the
//  Gnuplot executable (either MS-DOS or Windows executable) will not open.
//  If you have an idea to solve this, please E-mail me.
//
//
//  Feel free to contact me when:
//  - YOU can get the program to run under Windows
//  - you encounter a compile error
//  - you have made additional functionality
//  - you have outsmarted me on this interface.
//

#ifndef __GNUPLOT_H
#define __GNUPLOT_H

#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <vector>
#include <cstdio>
#include <ctime>
#include <cassert>

enum GnuplotStyle
{
  styleLines,
  stylePoints,
  styleLinespoints,
  styleImpulses,
  styleDots,
  styleSteps,
  styleErrorbars,
  styleBoxes,
  styleBoxerrorbars,
};

enum GnuplotTerminal
{
  terminalX11,
  terminalAqua
};

struct WindowData
{
  std::vector<std::string> tempFileName;
};

struct Gnuplot
{
  Gnuplot();
  ~Gnuplot();

  void execute(const std::string&) const;

  void addWindow(const int& nWindows = 1);
  void changeWindow(const int&);

  void setTerminal(const GnuplotTerminal&);
  void setStyle(const GnuplotStyle&);
  void setYlabel(const std::string&);
  void setXlabel(const std::string&);

  void plot(const std::string &, const std::string& title = "y");
  void plot(const std::vector<double>&, const std::string& title = "y");
  void plot(const std::vector<double>&, const std::vector<double>&, const std::string& title = "data");
  void plot(const std::vector<double>&, const std::vector<double>&, const std::vector<double>&, const std::string& title = "z");

  void emptyPlot();

  //Helper functions
  void sleep(const double& ) const;
  std::string itoa(const int&) const;
  std::string ftoa(const double&) const;
  bool fileExists(const std::string& fileName) const;

  private:
  std::FILE * mGnuPipe;
  std::string mTerminal;
  std::string mStyle;
  std::vector<WindowData> mWindowData;
  int mCurrentWindowNumber;
  static int mTempFileNumber;

  void setLineStyles();
  std::string readFileName() const;
  std::string createTempFileName();
  void createTempFile(const std::string&, const std::vector<double>&);
  void createTempFile(const std::string&, const std::vector<double>&, const std::vector<double>&);
  void createTempFile(const std::string&, const std::vector<double>&, const std::vector<double>&, const std::vector<double>&);
  void plotTempFile2D(const std::string&, const std::string&);
  void plotTempFile3D(const std::string&, const std::string&);
  void dot(const std::string& debugString = ".") const;
};

#endif


